@@ -3,6 +3,31 @@
 # Version 1.0
 # License GPL v3
 
+
+#setMethod("mosaic", signature(x="SpatRaster", y="SpatRaster"), 
+	.mosaic <- function(x, y, ..., filename="", overwrite=FALSE, wopt=list()) { 
+		opt <- .runOptions(filename, overwrite, wopt)
+		dots <- list(...)
+		rc <- SpatRasterCollection$new()		
+		rc$add(x@ptr)
+		rc$add(y@ptr)
+		n <- length(dots)
+		if (n > 0) {
+			for (i in 1:n) {
+				if (inherits(dots[[i]], "SpatRaster")) {
+					rc$add(dots[[i]]@ptr)
+				} else {
+					name <- names(dots[[i]])
+					cls <- class(dots[[i]])
+					stop(paste("additional arguments should be of class 'SpatRaster'\n Found argument", name, "of class: ", cls))
+				}
+			}
+		}
+		x@ptr <- rc$mosaic(opt)
+		show_messages(x, "mosaic")		
+	}
+#)
+
 setMethod("merge", signature(x="SpatRaster", y="SpatRaster"), 
 	function(x, y, ..., filename="", overwrite=FALSE, wopt=list()) { 
 		opt <- .runOptions(filename, overwrite, wopt)
