@@ -3,7 +3,6 @@
 # Version 1.0
 # License GPL v3
 
-
 setMethod("names", signature(x="SpatRaster"), 
 	function(x) { 
 		x@ptr$names
@@ -11,6 +10,7 @@ setMethod("names", signature(x="SpatRaster"),
 )
 
 
+
 setMethod("names<-", signature(x="SpatRaster"), 
 	function(x, value)  {
 		value <- as.character(value)
@@ -63,3 +63,25 @@ setMethod("names<-", signature(x="SpatVector"),
 		return(x)
 	}
 )
+
+
+setMethod("lnames", signature(x="SpatRaster"), 
+	function(x) { 
+		x@ptr$long_names
+	}
+)
+
+
+setMethod("lnames<-", signature(x="SpatRaster"), 
+	function(x, value)  {
+		value <- as.character(value)
+		if (length(value) != nlyr(x)) {
+			stop("incorrect number of names")
+		}
+		if (! x@ptr$set_long_names(value)) {
+			stop("cannot set these names")
+		}
+		return(x)
+	}
+)
+
