@@ -12,16 +12,16 @@
 #	mar + dmar
 #}
 
-.legMain <- function(leg.main, xmax, ymax, dy, leg.main.cex) {
-    if (!is.null(leg.main)) {
-		n <- length(leg.main)
-		ymax <- ymax + 0.05 * dy
-		for (i in 1:n) {
-			text(x=xmax, y=ymax+(n-i)*0.05*dy,
-				labels = leg.main[i], cex = leg.main.cex, xpd=TRUE)
-		}
-	}
-}
+# .legMain <- function(leg.main, xmax, ymax, dy, leg.main.cex) {
+    # if (!is.null(leg.main)) {
+		# n <- length(leg.main)
+		# ymax <- ymax + 0.05 * dy
+		# for (i in 1:n) {
+			# text(x=xmax, y=ymax+(n-i)*0.05*dy,
+				# labels = leg.main[i], cex = leg.main.cex, xpd=TRUE)
+		# }
+	# }
+# }
 
 .getLegCoords <- function(p, ext, leg.shrink, leg.main) {
 
