@@ -32,11 +32,12 @@ function(x, r=1, g=2, b=3, scale, maxcell=500000, stretch=NULL, ext=NULL, interp
 	}
 	scale <- as.vector(scale)[1]
 	
-	r <- spatSample(x[[r]], maxcell, ext=ext, method="regular", as.raster=TRUE)
-	g <- spatSample(x[[g]], maxcell, ext=ext, method="regular", as.raster=TRUE)
-	b <- spatSample(x[[b]], maxcell, ext=ext, method="regular", as.raster=TRUE)
+	if (!is.null(ext)) {
+		x <- crop(x, ext)
+	}
+	x <- spatSample(x[[c(r, g, b)]], maxcell, method="regular", as.raster=TRUE)
 
-	RGB <- cbind(values(r), values(g), values(b))
+	RGB <- values(x)
 	
 	if (!is.null(zlim)) {
 		if (length(zlim) == 2) {
@@ -87,23 +88,25 @@ function(x, r=1, g=2, b=3, scale, maxcell=500000, stretch=NULL, ext=NULL, interp
 	if (!is.null(naind)) {
 		bg <- grDevices::col2rgb(colNA)
 		bg <- grDevices::rgb(bg[1], bg[2], bg[3], alpha=bgalpha, max=255)
-		z <- rep( bg, times=ncell(r))
+		z <- rep( bg, times=ncell(x))
 		z[-naind] <- grDevices::rgb(RGB[,1], RGB[,2], RGB[,3], alpha=alpha, max=scale)
 	} else {
 		z <- grDevices::rgb(RGB[,1], RGB[,2], RGB[,3], alpha=alpha, max=scale)
 	}
 	
-	z <- matrix(z, nrow=nrow(r), ncol=ncol(r), byrow=T)
+	z <- matrix(z, nrow=nrow(x), ncol=ncol(x), byrow=TRUE)
 
 	requireNamespace("grDevices")
 	bb <- as.vector(matrix(as.vector(ext(x)), ncol=2))
 
-	bb <- as.vector(ext(r))
+	bb <- as.vector(ext(x))
 	
 	if (!add) {
-		if ((!axes) & (!margins)) {
-			graphics::par(plt=c(0,1,0,1))
-		}
+		#if ((!axes) & (!margins)) {
+		#	old.par <- graphics::par(no.readonly =TRUE)
+		#	on.exit(graphics::par(old.par))   
+		#	graphics::par(plt=c(0,1,0,1))
+		#}
 
 		if (is.null(asp)) {
 			if (isLonLat(x, perhaps=TRUE, warn=FALSE)) {
@@ -122,8 +125,9 @@ function(x, r=1, g=2, b=3, scale, maxcell=500000, stretch=NULL, ext=NULL, interp
 		if (axes) {
 			xticks <- graphics::axTicks(1, c(xlim[1], xlim[2], 4))
 			yticks <- graphics::axTicks(2, c(ylim[1], ylim[2], 4))
-			if (xres(r) %% 1 == 0) xticks = round(xticks)
-			if (yres(r) %% 1 == 0) yticks = round(yticks)
+			
+			if (xres(x) %% 1 == 0) xticks = round(xticks)
+			if (yres(x) %% 1 == 0) yticks = round(yticks)
 			graphics::axis(1, at=xticks)
 			graphics::axis(2, at=yticks, las = 1)
 			#graphics::axis(3, at=xticks, labels=FALSE, lwd.ticks=0)
