@@ -9,10 +9,33 @@ setMethod("readStart", signature(x="SpatRaster"),
 	function(x, ...) {
 		success <- x@ptr$readStart()
 		show_messages(x, "readStart")		
+		if (!success) stop("cannot open file for reading")
 		invisible(success)
 	}
 )
 
+setMethod("readStart", signature(x="SpatDataSet"), 
+	function(x, ...) {
+		success <- x@ptr$readStart()
+		show_messages(x, "readStart")		
+		if (!success) stop("cannot open file for reading")
+		invisible(success)
+	}
+)
+
+#setMethod("readStart", signature(x="SpatDataSet"), 
+#	function(x, ...) {
+#		nsd <- length(x)
+#		for (i in 1:nsd) {
+#			y <- x[i]
+#			success <- readStart(y)
+#			x[i] <- y
+#		}
+#		show_messages(x, "readStart")		
+#		invisible(success)
+#	}
+#)
+
 
 setMethod("readStop", signature(x="SpatRaster"), 
 	function(x) {
@@ -22,13 +45,21 @@ setMethod("readStop", signature(x="SpatRaster"),
 	}
 )
 
+setMethod("readStop", signature(x="SpatDataSet"), 
+	function(x) {
+		success <- x@ptr$readStop()
+		show_messages(x, "readStop")		
+		invisible(success)
+	}
+)
+
 
 setMethod("writeStart", signature(x="SpatRaster", filename="character"), 
 	function(x, filename="", overwrite=FALSE, wopt=list(), ...) {
 		opt <- .runOptions(filename, overwrite, wopt)
 		ok <- x@ptr$writeStart(opt)
 		show_messages(x, "writeStart")		
-		b <- x@ptr$getBlockSize(4)
+		b <- x@ptr$getBlockSize(4, opt$memfrac)
 		b$row <- b$row + 1
 		b		
 	}
