@@ -293,7 +293,10 @@ SpatRaster SpatRaster::aggregate(std::vector<unsigned> fact, std::string fun, bo
 	if (!out.writeStart(opt)) { return out; }
 
 	size_t nc = ncol();
-	readStart();
+	if (!readStart()) {
+		out.setError(getError());
+		return(out);
+	}
 	for (size_t i = 0; i < bs.n; i++) {
         std::vector<double> vin = readValues(bs.row[i], bs.nrows[i], 0, nc);
 		std::vector<double> v  = compute_aggregates(vin, bs.nrows[i], nc, nlyr(), fact, agFun, narm);
