@@ -18,6 +18,9 @@
 #include "spatRaster.h"
 #include "string_utils.h"
 #include "file_utils.h"
+#include "time.h"
+#include "recycle.h"
+
 #include <set>
 
 #ifdef useGDAL
@@ -437,6 +440,10 @@ std::vector<std::string> SpatRaster::getNames() {
 
 
 bool SpatRaster::setNames(std::vector<std::string> names, bool make_valid) {
+	if (names.size() == 1) {
+		recycle(names, nlyr());
+	}
+
 	if (names.size() != nlyr()) {
 		return false;
 	} else {
@@ -456,20 +463,86 @@ bool SpatRaster::setNames(std::vector<std::string> names, bool make_valid) {
 }
 
 
-std::vector<bool> SpatRaster::hasTime() {
-	std::vector<bool> x(source.size());
+std::vector<std::string> SpatRaster::getLongNames() {
+	std::vector<std::string> x;
 	for (size_t i=0; i<source.size(); i++) {
-		x[i] = source[i].hasTime; 
+		x.insert(x.end(), source[i].long_names.begin(), source[i].long_names.end());
 	}
 	return(x);
 }
 
 
-std::vector<double> SpatRaster::getTime() {
-	std::vector<double> x;
+bool SpatRaster::setLongNames(std::vector<std::string> nms) {
+	if (nms.size() == 1) {
+        size_t begin=0;
+        for (size_t i=0; i<source.size(); i++)	{
+            size_t end = begin + source[i].nlyr;
+			size_t sz =  end - begin + 1;
+            source[i].long_names = std::vector<std::string> (sz, nms[0]);
+            begin = end;
+        }
+        return true;
+	} else if (nms.size() != nlyr()) {
+		return false;
+	} else {
+        size_t begin=0;
+        size_t end;
+        for (size_t i=0; i<source.size(); i++)	{
+            end = begin + source[i].nlyr;
+            source[i].long_names = std::vector<std::string> (nms.begin() + begin, nms.begin() + end);
+            begin = end;
+        }
+        return true;
+	}
+}
+
+
+
+bool SpatRaster::hasTime() {
+	bool test = true;
+	for (size_t i=0; i<source.size(); i++) {
+		test = test & source[i].hasTime; 
+	}
+	return(test);
+}
+
+/*
+std::vector<double> SpatRaster::getTimeDbl() {
+	std::vector<int_64> t64 = getTime();
+	std::vector<double> out(t64.size());
+	for (size_t i=0; i < out.size(); i++) {
+		out[i] = t64[i];
+	}
+	return out;
+}
+*/
+
+std::vector<std::string> SpatRaster::getTimeStr() {
+	std::vector<std::string> out;
+	if (source[0].timestep == "seconds") {
+		std::vector<int_64> time = getTime();
+		out.reserve(time.size());
+		for (size_t i=0; i < out.size(); i++) {
+			std::vector<int> x = get_date(time[i]);
+			if (x.size() > 2) {
+				out.push_back( std::to_string(x[0]) + "-" 
+						  + std::to_string(x[1]) + "-"
+						  + std::to_string(x[2]) );
+						  
+			} else {
+				out.push_back("");
+			}
+		}
+	} 
+	return out;
+}
+
+
+std::vector<int_64> SpatRaster::getTime() {
+	std::vector<int_64> x;
 	for (size_t i=0; i<source.size(); i++) {
 		if (source[i].time.size() != source[i].nlyr) {
-			std::vector<double> nas(source[i].nlyr, NAN);
+			std::vector<double> nas(source[i].nlyr, 0);
 			x.insert(x.end(), nas.begin(), nas.end());			
 		} else {
 			x.insert(x.end(), source[i].time.begin(), source[i].time.end());
@@ -478,22 +551,30 @@ std::vector<double> SpatRaster::getTime() {
 	return(x);
 }
 
-
-bool SpatRaster::setTime(std::vector<double> times) {
-	if (times.size() != nlyr()) {
-		return false;
-	} else {
-        size_t begin=0;
-        size_t end;
-        for (size_t i=0; i<source.size(); i++)	{
-            end = begin + source[i].nlyr;
-            source[i].time = std::vector<double> (times.begin() + begin, times.begin() + end);
-            begin = end;
-        }
-        return true;
-	}
+std::string SpatRaster::getTimeStep() {
+	return source[0].timestep;
 }
 
+bool SpatRaster::setTime(std::vector<int_64> time, std::string step) {
+	if (time.size() != nlyr()) {
+		return false;
+	} 
+	if (!(step == "seconds") || (step == "raw")) {  // "days", "months", "years"
+		return false;
+	} 
+	size_t begin=0;
+	for (size_t i=0; i<source.size(); i++)	{
+		size_t end = begin + source[i].nlyr;
+        source[i].time = std::vector<int_64> (time.begin() + begin, time.begin() + end);
+		source[i].timestep = step;
+		source[i].hasTime = true;
+        begin = end;
+    }
+	
+    return true;
+}
+
+
 std::vector<double> SpatRaster::getDepth() {
 	std::vector<double> x;
 	for (size_t i=0; i<source.size(); i++) {
@@ -510,13 +591,17 @@ std::vector<double> SpatRaster::getDepth() {
 
 
 bool SpatRaster::setDepth(std::vector<double> depths) {
-	if (depths.size() != nlyr()) {
+	if (depths.size() == 1) {
+        for (size_t i=0; i<source.size(); i++)	{
+            source[i].depth = std::vector<double> (source[i].nlyr, depths[0]);
+        }
+        return true;
+	} else if (depths.size() != nlyr()) {
 		return false;
 	} else {
         size_t begin=0;
-        size_t end;
         for (size_t i=0; i<source.size(); i++)	{
-            end = begin + source[i].nlyr;
+            size_t end = begin + source[i].nlyr;
             source[i].depth = std::vector<double> (depths.begin() + begin, depths.begin() + end);
             begin = end;
         }
@@ -528,21 +613,16 @@ bool SpatRaster::setDepth(std::vector<double> depths) {
 
 bool SpatRaster::setUnit(std::vector<std::string> units) {
 	if (units.size() == 1) {
-        size_t begin=0;
         for (size_t i=0; i<source.size(); i++)	{
-            size_t end = begin + source[i].nlyr;
-			size_t sz =  end - begin + 1;
-            source[i].unit = std::vector<std::string> (sz, units[0]);
-            begin = end;
+            source[i].unit = std::vector<std::string> (source[i].nlyr, units[0]);
         }
         return true;
 	} else if (units.size() != nlyr()) {
 		return false;
 	} else {
         size_t begin=0;
-        size_t end;
         for (size_t i=0; i<source.size(); i++)	{
-            end = begin + source[i].nlyr;
+            size_t end = begin + source[i].nlyr;
             source[i].unit = std::vector<std::string> (units.begin() + begin, units.begin() + end);
             begin = end;
         }
@@ -565,6 +645,8 @@ std::vector<std::string> SpatRaster::getUnit() {
 }
 
 
+
+
 double SpatRaster::xres() {
 	SpatExtent extent = getExtent();
 	return (extent.xmax - extent.xmin) / ncol() ;
