@@ -268,6 +268,9 @@ unsigned SpatVector::nxy() {
 	unsigned n = 0;
 	for (size_t i=0; i < size(); i++) {
 		SpatGeom g = getGeom(i);
+		if (g.size() == 0) {
+			n++; // empty
+		}
 		for (size_t j=0; j < g.size(); j++) {
 			SpatPart p = g.getPart(j);
 			n += p.x.size();
@@ -308,7 +311,6 @@ std::vector<std::vector<double>> SpatVector::coordinates() {
 }
 
 
-
 SpatDataFrame SpatVector::getGeometryDF() {
 
 	SpatDataFrame out;
@@ -324,6 +326,15 @@ SpatDataFrame SpatVector::getGeometryDF() {
 	size_t idx = 0;
 	for (size_t i=0; i < size(); i++) {
 		SpatGeom g = getGeom(i);
+		if (g.size() == 0) { // empty
+			out.iv[0][idx] = i+1;
+			out.iv[1][idx] = 1;
+			out.dv[0][idx] = NAN;
+			out.dv[1][idx] = NAN;
+			out.iv[2][idx] = 0;
+			idx++;
+		}
+
 		for (size_t j=0; j < g.size(); j++) {
 			SpatPart p = g.getPart(j);
 			for (size_t q=0; q < p.x.size(); q++) {
@@ -352,6 +363,86 @@ SpatDataFrame SpatVector::getGeometryDF() {
 	return out;
 }
 
+std::string nice_string(const double &x) {
+	std::string s = std::to_string(x);
+	s.erase(s.find_last_not_of('0') + 1, std::string::npos);
+	s.erase(s.find_last_not_of('.') + 1, std::string::npos);
+	return s;
+}
+
+std::vector<std::string> SpatVector::getGeometryWKT() {
+
+	std::vector<std::string> out(size());
+	std::string wkt;
+	for (size_t i=0; i < size(); i++) {
+		SpatGeom g = getGeom(i);
+		size_t n = g.size();
+		if (g.gtype == points) {
+			if (n > 1) {
+				wkt = "MULTIPOINT ";
+			} else {
+				wkt = "POINT ";				
+			}
+		} else if (g.gtype == lines) {
+			if (n > 1) {
+				wkt = "MULTILINESTRING ";
+			} else {
+				wkt = "LINESTRING ";		
+			}
+		} else if (g.gtype == polygons) {
+			if (n > 1) {
+				wkt = "MULTIPOLYGON ";
+			} else {
+				wkt = "POLYGON ";
+			}
+		}
+		
+		if (n == 0) {
+			wkt += "EMPTY";
+			out[i] = wkt;
+			continue;
+		}
+
+		if ((g.gtype == polygons) | (n > 1)) { 
+			wkt += "(";
+		}			
+		
+		for (size_t j=0; j < n; j++) {
+			SpatPart p = g.getPart(j);
+			if (j>0) wkt += ",";
+
+			if ((g.gtype == polygons) & (n > 1)) { 
+				wkt += "(";
+			}			
+		
+			wkt += "(" + nice_string(p.x[0]) + " " + nice_string(p.y[0]);
+			for (size_t q=1; q < p.x.size(); q++) {
+				wkt += ", " + nice_string(p.x[q]) + " " + nice_string(p.y[q]);
+			}
+			wkt += ")";
+			if (p.hasHoles()) {
+				for (size_t k=0; k < p.nHoles(); k++) {
+					SpatHole h = p.getHole(k);
+					wkt += ",(" + nice_string(h.x[0]) + " " + nice_string(h.y[0]);
+					for (size_t q=1; q < h.x.size(); q++) {
+						wkt += ", " + nice_string(h.x[q]) + " " + nice_string(h.y[q]);
+					}
+					wkt += ")";
+				}
+			}
+			if ((g.gtype == polygons) & (n > 1)) { 
+				wkt += ")";
+			}			
+		}
+		if ((g.gtype == polygons) | (n > 1)) {
+			wkt += ")";
+		}
+		out[i] = wkt;
+	}
+	return out;
+}
+
+
 SpatGeomType SpatVector::getGType(std::string &type) {
 	if (type == "points") { return points; }
 	else if (type == "lines") { return lines; }
